#!/usr/bin/env ruby

require "fox"
require "fox/responder"

include Fox

class ButtonWindow < FXMainWindow

  include Responder

  # Message identifiers for this class
  ID_ICON_BEFORE_TEXT, ID_ICON_AFTER_TEXT, ID_ICON_CENTER_HOR,
  ID_ICON_ABOVE_TEXT, ID_ICON_BELOW_TEXT, ID_ICON_CENTER_VER,
  ID_JUST_CENTER_X, ID_JUST_LEFT, ID_JUST_RIGHT, ID_JUST_HOR_APART,
  ID_JUST_CENTER_Y, ID_JUST_TOP, ID_JUST_BOTTOM, ID_JUST_VER_APART,
  ID_TOOLBAR_STYLE, ID_QUIT = enum(FXMainWindow::ID_LAST, 16)


  def initialize(app)
    # Invoke base class initialize first
    super(app, "Button Test", nil, nil, DECOR_ALL, 100, 100, 0, 0)

    # Define the message map for the class
    FXMAPFUNC(SEL_COMMAND, ID_ICON_BEFORE_TEXT, "onCmdIconBeforeText")
    FXMAPFUNC(SEL_COMMAND, ID_ICON_AFTER_TEXT,  "onCmdIconAfterText")
    FXMAPFUNC(SEL_COMMAND, ID_ICON_CENTER_HOR,  "onCmdIconCenterHor")

    FXMAPFUNC(SEL_COMMAND, ID_ICON_ABOVE_TEXT,  "onCmdIconAboveText")
    FXMAPFUNC(SEL_COMMAND, ID_ICON_BELOW_TEXT,  "onCmdIconBelowText")
    FXMAPFUNC(SEL_COMMAND, ID_ICON_CENTER_VER,  "onCmdIconCenterVer")

    FXMAPFUNC(SEL_COMMAND, ID_JUST_CENTER_X,    "onCmdJustCenterX")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_LEFT,        "onCmdJustLeft")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_RIGHT,       "onCmdJustRight")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_HOR_APART,   "onCmdJustHorApart")

    FXMAPFUNC(SEL_COMMAND, ID_JUST_CENTER_Y,    "onCmdJustCenterY")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_TOP,         "onCmdJustTop")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_BOTTOM,      "onCmdJustBottom")
    FXMAPFUNC(SEL_COMMAND, ID_JUST_VER_APART,   "onCmdJustVerApart")

    FXMAPFUNC(SEL_COMMAND, ID_TOOLBAR_STYLE,    "onCmdToolbarStyle")
    FXMAPFUNC(SEL_COMMAND, ID_QUIT,             "onCmdQuit")

    # Create a tooltip
    FXTooltip.new(self.getApp())

    # Status bar
    statusbar = FXStatusbar.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)

    # Controls on the right
    controls = FXVerticalFrame.new(self,
      LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH)

    # Separator
    FXVerticalSeparator.new(self,
      LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE)

    # Contents
    contents = FXHorizontalFrame.new(self,
      LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH,
      0, 0, 0, 0, 20, 20, 20, 20)

    # Construct icon from a GIF file on disk
    bigpenguin = loadIcon("bigpenguin.gif")

    # The button
    @button = FXButton.new(contents,
      "&This is a multi-line label on\na button to show off the full\n" +
      "capabilities of the button object.\t" +
      "It also has a tooltip\n(which, by the way, can be multi-line also).\t" +
      "Here's a helpful message for the status line.",
      bigpenguin,
      nil, 0,
      FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,
      0, 0, 300, 200)

    FXCheckButton.new(controls,
      "Toolbar Style\tCool \"poppy\" style buttons", self, ID_TOOLBAR_STYLE)

    group1 = FXGroupBox.new(controls, "Horizontal Placement",
      GROUPBOX_TITLE_CENTER|FRAME_RIDGE)
    FXRadioButton.new(group1, "Before Text", self, ID_ICON_BEFORE_TEXT)
    FXRadioButton.new(group1, "After Text",  self, ID_ICON_AFTER_TEXT)
    FXRadioButton.new(group1, "Centered",    self, ID_ICON_CENTER_HOR)

    group2 = FXGroupBox.new(controls, "Vertical Placement",
      GROUPBOX_TITLE_CENTER|FRAME_RIDGE)
    FXRadioButton.new(group2, "Above Text", self, ID_ICON_ABOVE_TEXT)
    FXRadioButton.new(group2, "Below Text", self, ID_ICON_BELOW_TEXT)
    FXRadioButton.new(group2, "Centered",   self, ID_ICON_CENTER_VER)

    group3 = FXGroupBox.new(controls, "Horizontal Justification",
      GROUPBOX_TITLE_CENTER|FRAME_RIDGE)
    FXRadioButton.new(group3, "Center", self, ID_JUST_CENTER_X)
    FXRadioButton.new(group3, "Left",   self, ID_JUST_LEFT)
    FXRadioButton.new(group3, "Right",  self, ID_JUST_RIGHT)
    FXRadioButton.new(group3, "Apart",  self, ID_JUST_HOR_APART)

    group4 = FXGroupBox.new(controls, "Vertical Justification",
      GROUPBOX_TITLE_CENTER|FRAME_RIDGE)
    FXRadioButton.new(group4, "Center", self, ID_JUST_CENTER_Y)
    FXRadioButton.new(group4, "Top",    self, ID_JUST_TOP)
    FXRadioButton.new(group4, "Bottom", self, ID_JUST_BOTTOM)
    FXRadioButton.new(group4, "Apart",  self, ID_JUST_VER_APART)

    FXButton.new(controls, "&Quit", nil, self, ID_QUIT,
      FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X)
  end

  def loadIcon(filename)
    FXGIFIcon.new(getApp(), File.open("icons/" + filename, "rb").read())
  end

  def create
    super
    show(PLACEMENT_SCREEN)
  end

  def onCmdIconBeforeText(sender, sel, ptr)
    @button.iconPosition =
      (@button.iconPosition|ICON_BEFORE_TEXT) & ~ICON_AFTER_TEXT
  end

  def onCmdIconAfterText(sender, sel, ptr)
    @button.iconPosition =
      (@button.iconPosition|ICON_AFTER_TEXT) & ~ICON_BEFORE_TEXT
  end

  def onCmdIconCenterHor(sender, sel, ptr)
    @button.iconPosition =
      (@button.iconPosition & ~ICON_AFTER_TEXT) & ~ICON_BEFORE_TEXT
  end

  def onCmdIconAboveText(sender, sel, ptr)
    @button.iconPosition = 
      (@button.iconPosition|ICON_ABOVE_TEXT) & ~ICON_BELOW_TEXT
  end

  def onCmdIconBelowText(sender, sel, ptr)
    @button.iconPosition =
      (@button.iconPosition|ICON_BELOW_TEXT) & ~ICON_ABOVE_TEXT
  end

  def onCmdIconCenterVer(sender, sel, ptr)
    @button.iconPosition = 
      (@button.iconPosition & ~ICON_ABOVE_TEXT) & ~ICON_BELOW_TEXT
  end

  def onCmdJustCenterX(sender, sel, ptr)
    @button.justify &= ~JUSTIFY_HZ_APART
  end

  def onCmdJustLeft(sender, sel, ptr)
    @button.justify = (@button.justify & ~JUSTIFY_HZ_APART) | JUSTIFY_LEFT
  end

  def onCmdJustRight(sender, sel, ptr)
    @button.justify = (@button.justify & ~JUSTIFY_HZ_APART) | JUSTIFY_RIGHT
  end

  def onCmdJustHorApart(sender, sel, ptr)
    @button.justify |= JUSTIFY_HZ_APART
  end

  def onCmdJustCenterY(sender, sel, ptr)
    @button.justify &= ~JUSTIFY_VT_APART
  end

  def onCmdJustTop(sender, sel, ptr)
    @button.justify = (@button.justify & ~JUSTIFY_VT_APART) | JUSTIFY_TOP
  end

  def onCmdJustBottom(sender, sel, ptr)
    @button.justify = (@button.justify & ~JUSTIFY_VT_APART) | JUSTIFY_BOTTOM
  end

  def onCmdJustVerApart(sender, sel, ptr)
    @button.justify |= JUSTIFY_VT_APART
  end

  def onCmdToolbarStyle(sender, sel, ptr)
    if ptr
      @button.buttonStyle |= BUTTON_TOOLBAR
      @button.frameStyle = FRAME_RAISED
    else
      @button.buttonStyle &= ~BUTTON_TOOLBAR
      @button.frameStyle = FRAME_RAISED|FRAME_THICK
    end
  end

  def onCmdQuit(sender, sel, ptr)
    getApp().exit(0)
  end
end

def run
  # Construct an application
  application = FXApp.new("Button", "FoxTest")

  # Initialize application and open the display
  application.init(ARGV)

  # Construct the main window
  ButtonWindow.new(application)

  # Create the application
  application.create

  # Run it
  application.run
end

run
