#!/usr/bin/env ruby

require 'fox'
require 'fox/responder'

include Fox

class HeaderWindow < FXMainWindow

  include Responder

  # Define message identifiers for this class
  ID_ABOUT, ID_HEADER, ID_TRACKING, ID_LAST = enum(FXMainWindow::ID_LAST, 4)

  # About
  def onCmdAbout(sender, sel, ptr)
    FXMessageBox.information(self, MBOX_OK, "About Header",
      "An example of how to work with the header control.")
    return 1
  end

  # User changed the width of one of the headers
  def onCmdHeader(sender, sel, which)
    @lists[which].setWidth(@header1.getItemSize(which))
  end

  # User clicked a header button
  def onCmdHeaderButton(sender, sel, which)
    numItems = @lists[which].getNumItems()
    for i in 0...numItems
      @lists[which].selectItem(i)
    end
    return 1
  end

  # Toggle continuous tracking mode
  def onCmdContinuous(sender, sel, ptr)
    @header1.headerStyle ^= HEADER_TRACKING
    @header2.headerStyle ^= HEADER_TRACKING
  end

  def create
    super
    show
  end

  def initialize(app)
    # Invoke base class initializer first
    super(app, "Header Control Test", nil, nil, DECOR_ALL, 0, 0, 800, 600)

    # Set up the message mapping
    FXMAPFUNC(SEL_COMMAND, ID_ABOUT,    "onCmdAbout")
    FXMAPFUNC(SEL_CHANGED, ID_HEADER,   "onCmdHeader")
    FXMAPFUNC(SEL_COMMAND, ID_HEADER,   "onCmdHeaderButton")
    FXMAPFUNC(SEL_COMMAND, ID_TRACKING, "onCmdContinuous")

    # Menu bar stretched along the top of the main window
    menubar = FXMenubar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

    # Status bar, stretched along the bottom
    FXStatusbar.new(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X)
  
    # File menu
    filemenu = FXMenuPane.new(self)
    FXMenuCommand.new(filemenu, "&Quit\tCtl-Q\tQuit the application", nil,
      getApp(), FXApp::ID_QUIT, MENU_DEFAULT)
    FXMenuTitle.new(menubar, "&File", nil, filemenu)

    # Help menu
    helpmenu = FXMenuPane.new(self)
    FXMenuCommand.new(helpmenu, "&About Header...", nil, self, ID_ABOUT)
    FXMenuTitle.new(menubar, "&Help", nil, helpmenu, LAYOUT_RIGHT)

    # Make Main Window contents
    contents = FXVerticalFrame.new(self,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0,0,0,0, 0,0,0,0, 0,0)
  
    # Make header control
    @header1 = FXHeader.new(contents, self, ID_HEADER,
      HEADER_BUTTON|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X)
  
    # Document icon
    doc = FXGIFIcon.new(getApp(), File.open("icons/minidoc.gif", "rb").read())
  
    @header1.appendItem("Name", doc, 150)
    @header1.appendItem("Type", nil, 140)
    @header1.appendItem("Layout Option", doc, 230)
    @header1.appendItem("Attributes", nil, 80)
 
    # Below header
    panes = FXHorizontalFrame.new(contents,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0,0,0,0, 0,0,0,0, 0,0)

    # Make 4 lists
    @lists = []
    @lists.push(FXList.new(panes, 1, nil, 0,
      LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LIST_BROWSESELECT, 0, 0, 150, 0))
    @lists.push(FXList.new(panes, 1, nil, 0,
      LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LIST_SINGLESELECT, 0, 0, 140, 0))
    @lists.push(FXList.new(panes, 1, nil, 0,
      LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LIST_MULTIPLESELECT, 0, 0, 230, 0))
    @lists.push(FXList.new(panes, 1, nil, 0,
      LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LIST_EXTENDEDSELECT, 0, 0, 80, 0))

    @lists[0].setBackColor(FXRGB(255, 240, 240))
    @lists[1].setBackColor(FXRGB(240, 255, 240))
    @lists[2].setBackColor(FXRGB(240, 240, 255))
    @lists[3].setBackColor(FXRGB(255, 255, 240))

    # Add some contents
    @lists[0].appendItem("Matsumoto Yukihiro")
    @lists[0].appendItem("Jeroen van der Zijp")
    @lists[0].appendItem("Lyle Johnson")
    @lists[0].appendItem("Andy Hunt")
    @lists[0].appendItem("Dave Thomas")
    @lists[0].appendItem("Charles Warren")

    @lists[1].appendItem("Father of Ruby")
    @lists[1].appendItem("Incorrigible Hacker")
    @lists[1].appendItem("Windows Hacker")
    @lists[1].appendItem("Pragmatic Hacker")
    @lists[1].appendItem("Ruby Hacker")
    @lists[1].appendItem("Shutter Hacker")

    @lists[2].appendItem("LAYOUT_FILL_X|LAYOUT_FILL_Y")
    @lists[2].appendItem("LAYOUT_FILL_Y")
    @lists[2].appendItem("LAYOUT_NORMAL")
    @lists[2].appendItem("LAYOUT_NORMAL")
    @lists[2].appendItem("LAYOUT_NORMAL")
    @lists[2].appendItem("LAYOUT_NORMAL")

    @lists[3].appendItem("A")
    @lists[3].appendItem("B")
    @lists[3].appendItem("C")
    @lists[3].appendItem("D")
    @lists[3].appendItem("E")
    @lists[3].appendItem("F")

    @header2 = FXHeader.new(panes, nil, 0,
      HEADER_VERTICAL|HEADER_BUTTON|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_Y)
    @header2.appendItem("Example", nil, 30)
    @header2.appendItem("Of", nil, 30)
    @header2.appendItem("Vertical", nil, 30)
    @header2.appendItem("Header", nil, 30)

    # Group box with some controls
    groupie = FXGroupBox.new(panes, "Controls",
      GROUPBOX_TITLE_CENTER|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    check = FXCheckButton.new(groupie,
      "Continuous Tracking\tContinuous\tTrack Header continuously",
      self, ID_TRACKING, ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)

    # Whip out a tooltip control, jeez, that's hard
    FXTooltip.new(getApp())
  end
end

def run
  # Construct a FOX application object
  application = FXApp.new("Header", "FoxTest")

  # Initialize it by passing in the command-line arguments
  application.init(ARGV)

  # Construct the main window
  HeaderWindow.new(application)

  # Create all the windows
  application.create()

  # Run the application
  application.run()
end

run
