#!/usr/bin/env ruby

require "fox"
require "fox/responder"

include Fox

class ImageWindow < FXMainWindow

  include Responder

  # Message identifiers for this class
  ID_ABOUT,
  ID_OPEN,
  ID_SAVE,
  ID_TITLE,
  ID_QUIT,
  ID_FILELIST,
  ID_RECENTFILE,
  ID_ROTATE_90,
  ID_ROTATE_180,
  ID_ROTATE_270,
  ID_MIRROR_HOR,
  ID_MIRROR_VER,
  ID_SCALE,
  ID_CROP,
  ID_LAST = enum(FXMainWindow::ID_LAST, 15)

  def initialize(app)
    # Invoke base class initialize first
    super(app, "FOX Image Viewer: - untitled", nil, nil, DECOR_ALL,
      0, 0, 850, 600, 0, 0)

    # Set up message map for this class
    FXMAPFUNC(SEL_COMMAND,       ID_ABOUT,      "onCmdAbout")
    FXMAPFUNC(SEL_COMMAND,       ID_OPEN,       "onCmdOpen")
    FXMAPFUNC(SEL_COMMAND,       ID_SAVE,       "onCmdSave")
    FXMAPFUNC(SEL_UPDATE,        ID_TITLE,      "onUpdTitle")
    FXMAPFUNC(SEL_COMMAND,       ID_QUIT,       "onCmdQuit")
    FXMAPFUNC(SEL_SIGNAL,        ID_QUIT,       "onCmdQuit")
    FXMAPFUNC(SEL_CLOSE,         ID_TITLE,      "onCmdQuit")
    FXMAPFUNC(SEL_DOUBLECLICKED, ID_FILELIST,   "onCmdFileList")
    FXMAPFUNC(SEL_COMMAND,       ID_RECENTFILE, "onCmdRecentFile")
    FXMAPFUNC(SEL_COMMAND,       ID_ROTATE_90,  "onCmdRotate")
    FXMAPFUNC(SEL_COMMAND,       ID_ROTATE_180, "onCmdRotate")
    FXMAPFUNC(SEL_COMMAND,       ID_ROTATE_270, "onCmdRotate")
    FXMAPFUNC(SEL_COMMAND,       ID_MIRROR_HOR, "onCmdMirror")
    FXMAPFUNC(SEL_COMMAND,       ID_MIRROR_VER, "onCmdMirror")
    FXMAPFUNC(SEL_COMMAND,       ID_SCALE,      "onCmdScale")
    FXMAPFUNC(SEL_COMMAND,       ID_CROP,       "onCmdCrop")
    FXMAPFUNC(SEL_UPDATE,        ID_ROTATE_90,  "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_ROTATE_180, "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_ROTATE_270, "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_MIRROR_HOR, "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_MIRROR_VER, "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_SCALE,      "onUpdImage")
    FXMAPFUNC(SEL_UPDATE,        ID_CROP,       "onUpdImage")

    # Recently used files list
    @mrufiles = FXRecentFiles.new()

    # Make some icons
    fileopenicon = getIcon("fileopen.gif")
    filesaveicon = getIcon("filesave.gif")
    cuticon = getIcon("cut.gif")
    copyicon = getIcon("copy.gif")
    pasteicon = getIcon("paste.gif")
    uplevelicon = getIcon("tbuplevel.gif")
    paletteicon = getIcon("colorpal.gif")

    # Make color dialog
    colordlg = FXColorDialog.new(self, "Color Dialog")
  
    # Make menu bar
    dragshell1 = FXToolbarShell.new(self, 0)
    menubar = FXMenubar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED)
    FXToolbarGrip.new(menubar, menubar, FXMenubar::ID_TOOLBARGRIP,
      TOOLBARGRIP_DOUBLE)
  
    # Tool bar
    dragshell2 = FXToolbarShell.new(self, 0)
    toolbar = FXToolbar.new(self, dragshell2, (LAYOUT_SIDE_TOP|
      PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|FRAME_RAISED|LAYOUT_FILL_X))
    FXToolbarGrip.new(toolbar, toolbar, FXToolbar::ID_TOOLBARGRIP,
      TOOLBARGRIP_DOUBLE)
  
    # Status bar
    statusbar = FXStatusbar.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)
  
    # File menu
    filemenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&File", nil, filemenu)
    
    # Edit Menu
    editmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&Edit", nil, editmenu)
  
    # Manpulation Menu
    manipmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar,"&Manipulation", nil, manipmenu)
  
    # View menu
    viewmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&View", nil, viewmenu)
    
    # Help menu
    helpmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&Help", nil, helpmenu, LAYOUT_RIGHT)

    # Splitter
    splitter = FXSplitter.new(self, (LAYOUT_SIDE_TOP|LAYOUT_FILL_X|
      LAYOUT_FILL_Y| SPLITTER_TRACKING|SPLITTER_VERTICAL|SPLITTER_REVERSED))
  
    # Sunken border for image widget
    imagebox = FXHorizontalFrame.new(splitter,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0)
  
    # Make image widget
    @imageview = FXImageView.new(imagebox, nil, nil, 0,
      LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    # Sunken border for file list
    @filebox = FXHorizontalFrame.new(splitter, LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0)
  
    # Make file list
    fileframe = FXHorizontalFrame.new(@filebox,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    @filelist = FXFileList.new(fileframe, self, ID_FILELIST,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE)
    FXButton.new(@filebox, "\tUp one level\tGo up to higher directory.",
      uplevelicon, @filelist, FXFileList::ID_DIRECTORY_UP,
      BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_FILL_Y)
  
    # Toobar buttons: File manipulation
    FXButton.new(toolbar, "&Open\tOpen Image\tOpen image file.", fileopenicon,
      self, ID_OPEN, ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)
    FXButton.new(toolbar, "&Save\tSave Image\tSave image file.", filesaveicon,
      self, ID_SAVE, ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)
  
    # Toobar buttons: Editing
    FXButton.new(toolbar, "Cut\tCut", cuticon, nil, 0,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)
    FXButton.new(toolbar, "Copy\tCopy", copyicon, nil, 0,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)
    FXButton.new(toolbar, "Paste\tPaste", pasteicon, nil, 0,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)
  
    # Color
    FXButton.new(toolbar, "&Colors\tColors\tDisplay color dialog.", paletteicon,
      colordlg, FXWindow::ID_SHOW,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_RIGHT)
  
    # File Menu entries
    FXMenuCommand.new(filemenu, "&Open...\tCtl-O\tOpen image file.",
      fileopenicon, self, ID_OPEN)
    FXMenuCommand.new(filemenu, "&Save...\tCtl-S\tSave image file.",
      filesaveicon, self, ID_SAVE)
    FXMenuCommand.new(filemenu, "Dump", nil, getApp(), FXApp::ID_DUMP)
  
    # Recent file menu; this automatically hides if there are no files
    sep1 = FXMenuSeparator.new(filemenu)
    sep1.setTarget(@mrufiles)
    sep1.setSelector(FXRecentFiles::ID_ANYFILES)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_1)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_2)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_3)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_4)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_5)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_6)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_7)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_8)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_9)
    FXMenuCommand.new(filemenu, nil, nil, @mrufiles, FXRecentFiles::ID_FILE_10)
    FXMenuCommand.new(filemenu, "&Clear Recent Files", nil,
      @mrufiles, FXRecentFiles::ID_CLEAR)
    sep2 = FXMenuSeparator.new(filemenu)
    sep2.setTarget(@mrufiles)
    sep2.setSelector(FXRecentFiles::ID_ANYFILES)
    FXMenuCommand.new(filemenu, "&Quit\tCtl-Q", nil,
      self, ID_QUIT, MENU_DEFAULT)
  
    # Edit Menu entries
    FXMenuCommand.new(editmenu, "&Undo\tCtl-Z\tUndo last change.")
    FXMenuCommand.new(editmenu, "&Redo\tCtl-R\tRedo last undo.")
    FXMenuCommand.new(editmenu, "&Copy\tCtl-C\tCopy selection to clipboard.",
      copyicon, nil, 0)
    FXMenuCommand.new(editmenu, "C&ut\tCtl-X\tCut selection to clipboard.",
      cuticon, nil, 0)
    FXMenuCommand.new(editmenu, "&Paste\tCtl-V\tPaste from clipboard.",
      pasteicon, nil, 0)
    FXMenuCommand.new(editmenu, "&Delete\t\tDelete selection.", nil, nil, 0)
  
    # Manipulation Menu entries
    FXMenuCommand.new(manipmenu, "Rotate 90\t\tRotate 90 degrees.", nil,
      self, ID_ROTATE_90)
    FXMenuCommand.new(manipmenu, "Rotate 180\t\tRotate 180 degrees.", nil,
      self, ID_ROTATE_180)
    FXMenuCommand.new(manipmenu, "Rotate -90\t\tRotate -90 degrees.", nil,
      self, ID_ROTATE_270)
    FXMenuCommand.new(manipmenu, "Mirror Hor.\t\tMirror Horizontally.", nil,
      self, ID_MIRROR_HOR)
    FXMenuCommand.new(manipmenu, "Mirror Ver.\t\tMirror Vertically.", nil,
      self, ID_MIRROR_VER)
    FXMenuCommand.new(manipmenu, "Scale...\t\tScale image.", nil,
      self, ID_SCALE)
    FXMenuCommand.new(manipmenu, "Crop...\t\tCrop image.", nil, self, ID_CROP)
  
    # View Menu entries
    FXMenuCommand.new(viewmenu, "File list\t\tDisplay file list.", nil,
      @filebox, FXWindow::ID_TOGGLESHOWN)
    FXMenuCommand.new(viewmenu,
      "Show hidden files\t\tShow hidden files and directories.", nil,
      @filelist, FXFileList::ID_TOGGLE_HIDDEN)
    FXMenuCommand.new(viewmenu,
      "Show small icons\t\tDisplay directory with small icons.", nil,
      @filelist, FXFileList::ID_SHOW_MINI_ICONS)
    FXMenuCommand.new(viewmenu,
      "Show big icons\t\tDisplay directory with big icons.", nil,
      @filelist, FXFileList::ID_SHOW_BIG_ICONS)
    FXMenuCommand.new(viewmenu,
      "Show details view\t\tDisplay detailed directory listing.", nil,
      @filelist, FXFileList::ID_SHOW_DETAILS)
    FXMenuCommand.new(viewmenu, "Rows of icons\t\tView row-wise.", nil,
      @filelist, FXFileList::ID_ARRANGE_BY_ROWS)
    FXMenuCommand.new(viewmenu, "Columns of icons\t\tView column-wise.", nil,
      @filelist,FXFileList::ID_ARRANGE_BY_COLUMNS)
    FXMenuCommand.new(viewmenu, "Toolbar\t\tDisplay toolbar.", nil,
      toolbar, FXWindow::ID_TOGGLESHOWN)
    FXMenuCommand.new(viewmenu, "Float toolbar\t\tUndock the toolbar.", nil,
      toolbar, FXToolbar::ID_UNDOCK)
    FXMenuCommand.new(viewmenu,
      "Dock toolbar top\t\tDock the toolbar on the top.", nil,
      toolbar, FXToolbar::ID_DOCK_TOP)
    FXMenuCommand.new(viewmenu,
      "Dock toolbar left\t\tDock the toolbar on the left.", nil,
      toolbar, FXToolbar::ID_DOCK_LEFT)
    FXMenuCommand.new(viewmenu,
      "Dock toolbar right\t\tDock the toolbar on the right.", nil,
      toolbar, FXToolbar::ID_DOCK_RIGHT)
    FXMenuCommand.new(viewmenu,
      "Dock toolbar bottom\t\tDock the toolbar on the bottom.", nil,
      toolbar, FXToolbar::ID_DOCK_BOTTOM)
    FXMenuCommand.new(viewmenu, "Status line\t\tDisplay status line.", nil,
      statusbar, FXWindow::ID_TOGGLESHOWN)
 
    # Help Menu entries
    FXMenuCommand.new(helpmenu, "&About FOX...", nil, self, ID_ABOUT, 0)
  
    # Make a tool tip
    FXTooltip.new(getApp(), TOOLTIP_NORMAL)
  
    # Recent files
    @mrufiles.setTarget(self)
    @mrufiles.setSelector(ID_RECENTFILE)

    # Initialize file name and pattern for file dialog
    @filename = "untitled"
    @preferredFileFilter = 0
  end

  # Convenience function to construct a GIF icon
  def getIcon(filename)
    filename = "icons/" + filename
    FXGIFIcon.new(getApp(), File.open(filename, "rb").read())
  end

  def hasExtension(filename, ext)
    File.basename(filename, ext) != File.basename(filename)
  end

  # Load the named image file
  def loadImage(file)
    img = nil
    if hasExtension(file, ".gif")
      img = FXGIFImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".bmp")
      img = FXBMPImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".xpm")
      img = FXXPMImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".png")
      img = FXPNGImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".jpg")
      img = FXJPEGImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".pcx")
      img = FXPCXImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".tif")
      img = FXTIFImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    end

    # Perhaps failed?
    if !img
      FXMessageBox.error(self, MBOX_OK, "Error loading image",
        "Unsupported type: #{ext}")
      return false
    end

    # Load it...
    stream = FXFileStream.new()
    if stream.open(file, FXStreamLoad)
      getApp().beginWaitCursor()
      img.loadPixels(stream)
      stream.close()
      img.create()
      old = @imageview.getImage()
      @imageview.setImage(img)
      old = nil
      getApp().endWaitCursor()
    end
    return true
  end

  # Save image to named file
  def saveImage(file)
    stream = FXFileStream.new()
    if stream.open(file, FXStreamSave)
      getApp().beginWaitCursor()
      img = @imageview.getImage()
      img.savePixels(stream)
      stream.close()
      getApp().endWaitCursor()
    end
    return true
  end

  # Display the "About..." dialog
  def onCmdAbout(sender, sel, ptr)
    FXMessageBox.new(self, "About Image Viewer",
      "Image Viewer demonstrates the FOX ImageView widget.\n\n" +
      "Using the FOX C++ GUI Library (http://www.cfdrc.com/FOX/fox.html)\n\n" +
      "Copyright (C) 2000 Jeroen van der Zijp (jvz@cfdrc.com)", nil,
      MBOX_OK|DECOR_TITLE|DECOR_BORDER).execute()
    return 1
  end

  # Open a new file
  def onCmdOpen(sender, sel, ptr)
    open = FXFileDialog.new(self, "Open Image")
    open.setFilename(@filename)
    patterns = ["All Files (*)",
                "GIF Image (*.gif)",
                "BMP Image (*.bmp)",
                "XPM Image (*.xpm)",
                "PNG Image (*.png)",
                "JPEG Image (*.jpg)",
                "PCX Image (*.pcx)",
                "TIFF Image (*.tif)"
    ]
    open.setPatternList(patterns)
    open.setCurrentPattern(@preferredFileFilter)
    if open.execute() != 0
      @preferredFileFilter = open.getCurrentPattern()
      @filename = open.getFilename()
      @filelist.setCurrentFile(@filename)
      @mrufiles.appendFile(@filename)
      loadImage(@filename)
    end
    return 1
  end

  # Save this file
  def onCmdSave(sender, sel, ptr)
    savedialog = FXFileDialog.new(self, "Save Image")
    if savedialog.execute() != 0
      if fxexists(savedialog.getFilename())
        if MBOX_CLICKED_NO == FXMessageBox.question(self, MBOX_YES_NO,
          "Overwrite Image", "Overwrite existing image?")
          return 1
        end
        @filename = savedialog.getFilename()
        @filelist.setCurrentFile(@filename)
        @mrufiles.appendFile(@filename)
        saveImage(@filename)
      end
    end
    return 1
  end

  # Quit the application
  def onCmdQuit(sender, sel, ptr)
    # Write new window size back to registry
    getApp().reg().writeIntEntry("SETTINGS", "x", getX())
    getApp().reg().writeIntEntry("SETTINGS", "y", getY())
    getApp().reg().writeIntEntry("SETTINGS", "width", getWidth())
    getApp().reg().writeIntEntry("SETTINGS", "height", getHeight())

    # Height of file list
    getApp().reg().writeIntEntry("SETTINGS", "fileheight", @filebox.getHeight())

    # Was file box shown?
    getApp().reg().writeIntEntry("SETTINGS", "filesshown",
      @filebox.shown() ? 1 : 0)

    # Current directory
    dir = @filelist.getDirectory()
    getApp().reg().writeStringEntry("SETTINGS", "directory", dir)

    # Quit
    getApp().exit(0)
  end

  # Update main window title
  def onUpdTitle(sender, sel, ptr)
    title = "FOX Image Viewer:- " + @filename
    image = @imageview.getImage()
    if image
      title += " (#{image.getWidth()} x #{image.getHeight})"
      sender.handle(self,
        MKUINT(FXWindow::ID_SETSTRINGVALUE, SEL_COMMAND), title)
    end
    return 1
  end

  # Command message from the file list
  def onCmdFileList(sender, sel, index)
    if index >= 0
      if @filelist.isItemDirectory(index)
        @filelist.setDirectory(@filelist.getItemPathname(index))
      elsif @filelist.isItemFile(index)
        @filename = @filelist.getItemPathname(index)
        @mrufiles.appendFile(@filename)
        loadImage(@filename)
      end
    end
    return 1
  end

  # Open recently used file
  def onCmdRecentFile(sender, sel, ptr)
    @filename = ptr
    @filelist.setCurrentFile(@filename)
    loadImage(@filename)
  end

  # Rotate image
  def onCmdRotate(sender, sel, ptr)
    image = @imageview.getImage()
    case SELID(sel)
      when ID_ROTATE_90 then image.rotate(90)
      when ID_ROTATE_180 then image.rotate(180)
      when ID_ROTATE_270 then image.rotate(270)
    end
    @imageview.setImage(image)
  end

  # Mirror
  def onCmdMirror(sender, sel, ptr)
    image = @imageview.getImage()
    case SELID(sel)
      when ID_MIRROR_HOR then image.mirror(true, false)
      when ID_MIRROR_VER then image.mirror(false, true)
    end
    @imageview.setImage(image)
  end

  # Scale
  def onCmdScale(sender, sel, ptr)
    image = @imageview.getImage()
    sx = FXDataTarget.new(image.getWidth())
    sy = FXDataTarget.new(image.getHeight())
    scalepanel = FXDialogBox.new(self, "Scale Image To Size")
    frame = FXHorizontalFrame.new(scalepanel,
      LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    FXLabel.new(frame, "W:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, sx, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXLabel.new(frame, "H:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, sy, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXButton.new(frame, "Cancel", nil, scalepanel, FXDialogBox::ID_CANCEL,
      LAYOUT_CENTER_Y|FRAME_RAISED|FRAME_THICK, 0, 0, 0, 0, 20, 20, 4, 4)
    FXButton.new(frame, "OK", nil, scalepanel, FXDialogBox::ID_ACCEPT,
      LAYOUT_CENTER_Y|FRAME_RAISED|FRAME_THICK, 0, 0, 0, 0, 30, 30, 4, 4)
    return 1 if (scalepanel.execute() == 0)
    return 1 if (sx.value < 1 || sy.value < 1)
    image.scale(sx.value, sy.value)
    @imageview.setImage(image)
  end

  # Crop
  def onCmdCrop(sender, sel, ptr)
    image = @imageview.getImage()
    cx = FXDataTarget.new(0)
    cy = FXDataTarget.new(0)
    cw = FXDataTarget.new(image.getWidth())
    ch = FXDataTarget.new(image.getHeight())
    croppanel = FXDialogBox.new(self, "Crop image")
    frame = FXHorizontalFrame.new(croppanel,
      LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    FXLabel.new(frame, "X:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, cx, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXLabel.new(frame, "Y:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, cy, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXLabel.new(frame, "W:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, cw, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXLabel.new(frame, "H:", nil, LAYOUT_CENTER_Y)
    FXTextField.new(frame, 5, ch, FXDataTarget::ID_VALUE,
      LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK|JUSTIFY_RIGHT)
    FXButton.new(frame, "Cancel", nil, croppanel, FXDialogBox::ID_CANCEL,
      LAYOUT_CENTER_Y|FRAME_RAISED|FRAME_THICK, 0, 0, 0, 0, 20, 20, 4, 4)
    FXButton.new(frame, "OK", nil, croppanel, FXDialogBox::ID_ACCEPT,
      LAYOUT_CENTER_Y|FRAME_RAISED|FRAME_THICK, 0, 0, 0, 0, 30, 30, 4, 4)
    return 1 if (croppanel.execute() == 0)
    return 1 if (cx.value < 0 || cy.value < 0 ||
        cx.value+cw.value > image.getWidth() ||
        cy.value+ch.value > image.getHeight())
    image.crop(cx.value, cy.value, cw.value, ch.value)
    @imageview.setImage(image)
  end

  # Update image
  def onUpdImage(sender, sel, ptr)
    if @imageview.getImage()
      sender.handle(self, MKUINT(FXWindow::ID_ENABLE, SEL_COMMAND), nil)
    else
      sender.handle(self, MKUINT(FXWindow::ID_DISABLE, SEL_COMMAND), nil)
    end
  end

  # Create and show window
  def create
    # Get size, etc. from registry
    xx = getApp().reg().readIntEntry("SETTINGS", "x", 0)
    yy = getApp().reg().readIntEntry("SETTINGS", "y", 0)
    ww = getApp().reg().readIntEntry("SETTINGS", "width", 850)
    hh = getApp().reg().readIntEntry("SETTINGS", "height", 600)

    fh = getApp().reg().readIntEntry("SETTINGS", "fileheight", 100)
    fs = getApp().reg().readIntEntry("SETTINGS", "filesshown", 1)

    dir = getApp().reg().readStringEntry("SETTINGS", "directory", "~")

    # Starting directory for the files list
    @filelist.setDirectory(dir)

    # Height of files list
    @filebox.setHeight(fh)

    # Is it visible right now?
    if fs != 1
      @filebox.hide()
    end
 
    # Reposition window to specified x, y, w and h
    position(xx, yy, ww, hh)

    # Create and show
    super()   # i.e. FXMainWindow::create()
    show()
  end
end

def run
  # Make application
  application = FXApp.new("ImageViewer", "FoxTest")

  # Open display
  application.init(ARGV)

  # Make window
  window = ImageWindow.new(application)

  # Handle interrupts to terminate program gracefully
  application.addSignal("SIGINT", window, ImageWindow::ID_QUIT)

  # Create it
  application.create()

  # Passed image file?
  if ARGV.length > 0
    window.loadImage(ARGV[0])
  end

  # Run
  application.run()
end

run
