#!/usr/bin/env ruby

require 'fox'
require 'fox/responder'

include Fox

class InputHandlerWindow < FXMainWindow

  include Responder

  # Message identifiers
  ID_PIPE, ID_TEXT = enum(FXMainWindow::ID_LAST, 2)

  def initialize(app)
    # Initialize base class first
    super(app, "Input Handlers Test", nil, nil, DECOR_ALL, 0, 0, 400, 300)

    # Message map
    FXMAPFUNC(SEL_COMMAND, ID_TEXT, "onCmdText")
    FXMAPFUNC(SEL_IO_READ, ID_PIPE, "onPipeRead")
    FXMAPFUNC(SEL_IO_EXCEPT, ID_PIPE, "onPipeException")

    # Text area plus a button
    commands = FXHorizontalFrame.new(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X)
    FXLabel.new(commands, "Command:")
    @cmdInput = FXTextField.new(commands, 30,
        self, ID_TEXT, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X) 
    FXHorizontalSeparator.new(self, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X)
    textFrame = FXVerticalFrame.new(self,
        FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y)

    # Output will be displayed in a multiline text area
    @cmdOutput = FXText.new(textFrame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y)

    # Initialize the pipe
    @pipe = nil
  end

  # Create and show the main window
  def create
    super
    show(PLACEMENT_SCREEN)
  end

  def onCmdText(sender, sel, ptr)
    # Remove previous input (if any)
    getApp().removeInput(@pipe, INPUT_READ|INPUT_EXCEPT) if @pipe

    # Clean up the output window
    @cmdOutput.text = ""

    # Open a new pipe
    @pipe = IO.popen(@cmdInput.text)

    # Register input callbacks and return
    getApp().addInput(@pipe, INPUT_READ|INPUT_EXCEPT, self, ID_PIPE)
    return 1
  end

  # This handler should get called when there's input waiting...
  def onPipeRead(sender, sel, ptr)
    latest = @pipe.read
#   @cmdOutput.appendText(latest, latest.length) if latest
    @cmdOutput.appendText(latest)
    return 1
  end

  # An exception occurred on the input
  def onPipeException(sender, sel, ptr)
    puts 'onPipeExcept'
    return 1
  end
end

if $0 == __FILE__
  # Construct an application
  application = FXApp.new('InputHandler', 'FoxTest')

  # Open the display
  application.init(ARGV)

  # Construct the main window
  InputHandlerWindow.new(application)

  # Create and show the application windows
  application.create

  # Run the application
  application.run
end
