#!/usr/bin/env ruby

require "fox"

include Fox

class ShutterWindow < FXMainWindow

  # This is just a helper function that loads a GIF file from disk
  # and constructs and returns a GIF icon object.
  def loadIcon(filename)
    FXGIFIcon.new(getApp(), File.open("icons/" + filename, "rb").read())
  end

  def initialize(app)
    # Invoke base class initialize first
    super(app, "Shutter Widget Test", nil, nil, DECOR_ALL, 0, 0, 600, 600)

    # Load up these icons
    foldericon   = loadIcon("shutter1.gif")
    compressicon = loadIcon("shutter2.gif")

    # Create top-level container
    contents = FXHorizontalFrame.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y)

    # List frame takes up most of the space on the left-hand side.
    # Since both the LAYOUT_FILL_X and LAYOUT_FILL_Y flags are set
    # it will expand to fill up any leftover space.
    listFrame = FXVerticalFrame.new(contents,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10)

    # Button frame appears on the right
    buttonFrame = FXVerticalFrame.new(contents,
      FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10)

    # The list frame contains a label, a separator and a tree list
    FXLabel.new(listFrame, "Tree List", nil, JUSTIFY_CENTER_X|LAYOUT_FILL_X)
    FXHorizontalSeparator.new(listFrame, SEPARATOR_GROOVE|LAYOUT_FILL_X)
    tree = FXTreeList.new(listFrame, 0, nil, 0,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_RIGHT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES)

    # The button frame also contains a label and a separator, plus the shutter
    FXLabel.new(buttonFrame, "Button Frame", nil,
      JUSTIFY_CENTER_X|LAYOUT_FILL_X)
    FXHorizontalSeparator.new(buttonFrame, SEPARATOR_RIDGE|LAYOUT_FILL_X)

    shutter = FXShutter.new(buttonFrame, nil, 0,
      FRAME_SUNKEN|LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    shutterItem = FXShutterItem.new(shutter, "Item 1", nil,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10, 10, 10)
    FXButton.new(shutterItem.getContent(), nil, foldericon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, compressicon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, compressicon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, foldericon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
  
    shutterItem = FXShutterItem.new(shutter, "Item 2", nil,
      LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10, 10, 10)
    FXButton.new(shutterItem.getContent(), nil, foldericon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, compressicon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, foldericon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
  
    shutterItem = FXShutterItem.new(shutter, "Item 3", nil,
      LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10, 10, 10)
    FXButton.new(shutterItem.getContent(), nil, foldericon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
    FXButton.new(shutterItem.getContent(), nil, compressicon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
  
    shutterItem = FXShutterItem.new(shutter, "Item 4", nil,
      LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 10, 10, 10, 10)
    FXButton.new(shutterItem.getContent(), nil, compressicon,
      getApp(), FXApp::ID_QUIT,
      FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 10, 10, 5, 5)
  end

  def create
    super
    show
  end
end

def run
  application = FXApp.new("Shutter", "FoxTest")
  application.init(ARGV)
  ShutterWindow.new(application)
  application.create
  application.run
end

run
