#!/usr/bin/env ruby

require "fox"
require "fox/responder"

include Fox

class TabBookWindow < FXMainWindow

  include Responder

  # Message identifiers for this class
  ID_TABS_TOP,
  ID_TABS_BOTTOM,
  ID_TABS_LEFT,
  ID_TABS_RIGHT,
  ID_HIDESHOW,
  ID_PANEL = enum(FXMainWindow::ID_LAST, 6)

  def initialize(app)
    # Call the base class initializer first
    super(app, "Tab Book Test", nil, nil, DECOR_ALL, 0, 0, 600, 400)

    # Define the message map for this class
    FXMAPFUNCS(SEL_COMMAND, ID_TABS_TOP, ID_TABS_RIGHT, "onCmdTabOrient")
    FXMAPFUNC(SEL_COMMAND, ID_HIDESHOW, "onCmdHideShow")
    FXMAPFUNC(SEL_COMMAND, ID_PANEL, "onCmdPanel")

    # Make a tooltip
    FXTooltip.new(getApp())

    # Menubar appears along the top of the main window
    menubar = FXMenubar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

    # Separator
    FXHorizontalSeparator.new(self,
      LAYOUT_SIDE_TOP|LAYOUT_FILL_X|SEPARATOR_GROOVE)

    # Contents
    contents = FXHorizontalFrame.new(self,
      LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH)
  
    # Switcher
    @tabbook = FXTabBook.new(contents, self, ID_PANEL,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
  
    # First item is a list
    @tab1 = FXTabItem.new(@tabbook, "&Simple List", nil)
    listframe = FXHorizontalFrame.new(@tabbook, FRAME_THICK|FRAME_RAISED)
    simplelist = FXList.new(listframe, 1, nil, 0,
      LIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    simplelist.appendItem("First Entry")
    simplelist.appendItem("Second Entry")
    simplelist.appendItem("Third Entry")
    simplelist.appendItem("Fourth Entry")
      
    # Second item is a file list
    @tab2 = FXTabItem.new(@tabbook, "F&ile List", nil)
    @fileframe = FXHorizontalFrame.new(@tabbook, FRAME_THICK|FRAME_RAISED)
    filelist = FXFileList.new(@fileframe, nil, 0,
      ICONLIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    # Third item is a directory list
    @tab3 = FXTabItem.new(@tabbook, "T&ree List", nil)
    dirframe = FXHorizontalFrame.new(@tabbook, FRAME_THICK|FRAME_RAISED)
    dirlist = FXDirList.new(dirframe, 0, nil, 0, (DIRLIST_SHOWFILES|
      TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|LAYOUT_FILL_X|LAYOUT_FILL_Y))
    
    # File Menu
    filemenu = FXMenuPane.new(self)
    FXMenuCommand.new(filemenu, "&Simple List", nil,
      @tabbook, FXTabBar::ID_OPEN_FIRST+0)
    FXMenuCommand.new(filemenu, "F&ile List", nil,
      @tabbook, FXTabBar::ID_OPEN_FIRST+1)
    FXMenuCommand.new(filemenu, "T&ree List", nil,
      @tabbook, FXTabBar::ID_OPEN_FIRST+2)
    FXMenuCommand.new(filemenu, "&Quit\tCtl-Q", nil,
      getApp(), FXApp::ID_QUIT)
    FXMenuTitle.new(menubar, "&File", nil, filemenu)
    
    # Tab side
    tabmenu = FXMenuPane.new(self)
    FXMenuCommand.new(tabmenu, "Hide/Show Tab 2", nil,
      self, TabBookWindow::ID_HIDESHOW)
    FXMenuCommand.new(tabmenu, "&Top Tabs", nil,
      self, TabBookWindow::ID_TABS_TOP)
    FXMenuCommand.new(tabmenu, "&Bottom Tabs", nil,
      self, TabBookWindow::ID_TABS_BOTTOM)
    FXMenuCommand.new(tabmenu, "&Left Tabs", nil,
      self, TabBookWindow::ID_TABS_LEFT)
    FXMenuCommand.new(tabmenu, "&Right Tabs", nil,
      self, TabBookWindow::ID_TABS_RIGHT)
    FXMenuTitle.new(menubar, "&Tab Placement", nil, tabmenu)
  end

  # Switch tab orientations
  def onCmdTabOrient(sender, sel, ptr)
    case SELID(sel)
      when ID_TABS_TOP
        @tabbook.setTabStyle(TABBOOK_TOPTABS)
        @tab1.setTabOrientation(TAB_TOP)
        @tab2.setTabOrientation(TAB_TOP)
        @tab3.setTabOrientation(TAB_TOP)
      when ID_TABS_BOTTOM
        @tabbook.setTabStyle(TABBOOK_BOTTOMTABS)
        @tab1.setTabOrientation(TAB_BOTTOM)
        @tab2.setTabOrientation(TAB_BOTTOM)
        @tab3.setTabOrientation(TAB_BOTTOM)
      when ID_TABS_LEFT
        @tabbook.setTabStyle(TABBOOK_LEFTTABS)
        @tab1.setTabOrientation(TAB_LEFT)
        @tab2.setTabOrientation(TAB_LEFT)
        @tab3.setTabOrientation(TAB_LEFT)
      when ID_TABS_RIGHT
        @tabbook.setTabStyle(TABBOOK_RIGHTTABS)
        @tab1.setTabOrientation(TAB_RIGHT)
        @tab2.setTabOrientation(TAB_RIGHT)
        @tab3.setTabOrientation(TAB_RIGHT)
    end
  end

  # Hide or show a panel
  def onCmdHideShow(sender, sel, ptr)
    if @tab2.shown
      @tab2.hide
      @fileframe.hide
    else
      @tab2.show
      @fileframe.show
    end
    @tab2.recalc
    @fileframe.recalc
  end

  # Active panel was switched
  def onCmdPanel(sender, sel, which)
    puts "Panel #{which} is now in front"
  end

  def create
    super
    show(PLACEMENT_SCREEN)
  end
end

def run
  # Make an application
  application = FXApp.new("TabBook", "FoxTest")

  # Open the display and read the registry
  application.init(ARGV)

  # Build the main window
  TabBookWindow.new(application)

  # Create the application and its windows
  application.create

  # Run
  application.run
end

run
