#!/usr/bin/env ruby

require "fox"
require "fox/responder"
require "date"

include Fox

class TableWindow < FXMainWindow

  include Responder

  # Define the message identifier for this class
  ID_RESIZETABLE = FXMainWindow::ID_LAST

  def initialize(app)
    # Call the base class initializer first
    super(app, "Table Widget Test", nil, nil, DECOR_ALL, 0, 0, 0, 0)

    # Set up the message map for this class
    FXMAPFUNC(SEL_COMMAND, ID_RESIZETABLE, "onCmdResizeTable")

    # Tooltip
    tooltip = FXTooltip.new(getApp())
    
    # Menubar
    menubar = FXMenubar.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
    
    # Separator
    FXHorizontalSeparator.new(self,
      LAYOUT_SIDE_TOP|LAYOUT_FILL_X|SEPARATOR_GROOVE)
  
    # Contents
    contents = FXVerticalFrame.new(self,
      LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    frame = FXVerticalFrame.new(contents,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
  
    # Table
    @table = FXTable.new(frame, 20, 8, nil, 0,
      TABLE_HOR_GRIDLINES|TABLE_VER_GRIDLINES|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0,0,0,0, 2,2,2,2)
    @table.setTableSize(50, 14)
    @table.setLeadingRows(1)
    @table.setLeadingCols(1)
    @table.setTrailingRows(1)
    @table.setTrailingCols(1)
  
    # Initialize first & last fixed rows
    (1..12).each { |c|
      @table.setItemText(0,  c, Date::MONTHNAMES[c])
      @table.setItemText(49, c, Date::MONTHNAMES[c])
    }
  
    # Initialize first/last fixed columns
    (1..48).each { |r|
      @table.setItemText(r,  0, r.to_s)
      @table.setItemText(r, 13, r.to_s)
    }
  
    # Initialize scrollable part of table
    (1..48).each { |r|
      (1..12).each { |c|
        @table.setItemText(r, c, "r:#{r} c:#{c}")
      }
    }
  
    # File Menu
    filemenu = FXMenuPane.new(self)
    FXMenuCommand.new(filemenu, "&Quit\tCtl-Q", nil, getApp(), FXApp::ID_QUIT)
    FXMenuTitle.new(menubar, "&File", nil, filemenu)
    
    # Options Menu
    tablemenu = FXMenuPane.new(self)
    FXMenuCommand.new(tablemenu, "Horizontal grid", nil,
      @table, FXTable::ID_HORZ_GRID)
    FXMenuCommand.new(tablemenu, "Vertical grid", nil,
      @table, FXTable::ID_VERT_GRID)
    FXMenuTitle.new(menubar, "&Options", nil, tablemenu)
    
    # Manipulations Menu
    manipmenu = FXMenuPane.new(self)
    FXMenuCommand.new(manipmenu, "Delete Column\tCtl-C", nil,
      @table, FXTable::ID_DELETE_COLUMN)
    FXMenuCommand.new(manipmenu, "Delete Row\tCtl-R", nil,
      @table, FXTable::ID_DELETE_ROW)
    FXMenuCommand.new(manipmenu, "Insert Column\tCtl-Shift-C", nil,
      @table, FXTable::ID_INSERT_COLUMN)
    FXMenuCommand.new(manipmenu, "Insert Row\tCtl-Shift-R", nil,
      @table, FXTable::ID_INSERT_ROW)
    FXMenuCommand.new(manipmenu, "Resize table...", nil,
      self, TableWindow::ID_RESIZETABLE)
    FXMenuTitle.new(menubar, "&Manipulations", nil, manipmenu)
    
    # Selection Menu
    selectmenu = FXMenuPane.new(self)
    FXMenuCommand.new(selectmenu, "Select All", nil,
      @table, FXTable::ID_SELECT_ALL)
    FXMenuCommand.new(selectmenu, "Select Cell", nil,
      @table, FXTable::ID_SELECT_CELL)
    FXMenuCommand.new(selectmenu, "Select Row", nil,
      @table, FXTable::ID_SELECT_ROW)
    FXMenuCommand.new(selectmenu, "Select Column", nil,
      @table, FXTable::ID_SELECT_COLUMN)
    FXMenuCommand.new(selectmenu, "Deselect All", nil,
      @table, FXTable::ID_DESELECT_ALL)
    FXMenuTitle.new(menubar, "&Selection", nil, selectmenu)
  end

  # Resize the table
  def onCmdResizeTable(sender, sel, ptr)
    # Create an empty dialog box
    dlg = FXDialogBox.new(self, "Resize Table")

    # Set up its contents
    frame = FXHorizontalFrame.new(dlg, LAYOUT_FILL_X|LAYOUT_FILL_Y)
    FXLabel.new(frame, "Rows:", nil, LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
    rows = FXTextField.new(frame, 5, nil, 0,
      JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
    FXLabel.new(frame, "Columns:", nil, LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
    cols = FXTextField.new(frame, 5, nil, 0,
      JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
    FXButton.new(frame, "Cancel", nil, dlg, FXDialogBox::ID_CANCEL,
      FRAME_RAISED|FRAME_THICK|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
    FXButton.new(frame, "  OK  ", nil, dlg, FXDialogBox::ID_ACCEPT,
      FRAME_RAISED|FRAME_THICK|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)

    # Initialize the text fields' contents
    oldnr, oldnc = @table.getNumRows(), @table.getNumCols()
    rows.setText(oldnr.to_s);
    cols.setText(oldnc.to_s);

    # FXDialogBox#execute will return non-zero if the user clicks OK
    if dlg.execute != 0
      nr, nc = rows.getText().to_i, cols.getText().to_i
      nr = 0  if nr < 0
      nc = 0  if nc < 0
      @table.setTableSize(nr, nc)
      (0...nr).each { |r|
        (0...nc).each { |c|
          if (r > oldnr) || (c > oldnc)
            @table.setItemText(r, c, "r:#{r} c:#{c}")
          end
        }
      }
    end
    return 1
  end

  # Create and show this window
  def create
    super
    show(PLACEMENT_SCREEN)
  end
end

# Start the whole thing
def run
  # Make application
  application = FXApp.new("TableApp", "FoxTest")
  
  # Open display
  application.init(ARGV)
  
  # Make window
  TableWindow.new(application)
  
  # Create app
  application.create
  
  # Run
  application.run
end

run
